Package: ASURAT
Type: Package
Title: Functional annotation-driven unsupervised clustering for
        single-cell data
Version: 1.14.0
Authors@R: c(person("Keita", "Iida", email = "kiida@protein.osaka-u.ac.jp",
             role = c("aut", "cre"),
             comment = c(ORCID = "0000-0002-1076-830X")),
             person("Johannes Nicolaus", "Wibisana",
                    email = "nico@protein.osaka-u.ac.jp", role = c("ctb")))
Description: ASURAT is a software for single-cell data analysis.
    Using ASURAT, one can simultaneously perform unsupervised clustering and
    biological interpretation in terms of cell type, disease, biological process,
    and signaling pathway activity. Inputting a single-cell RNA-seq data and
    knowledge-based databases, such as Cell Ontology, Gene Ontology, KEGG, etc.,
    ASURAT transforms gene expression tables into original multivariate tables,
    termed sign-by-sample matrices (SSMs).
License: GPL-3 + file LICENSE
biocViews: GeneExpression, SingleCell, Sequencing, Clustering,
        GeneSignaling
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: TRUE
Depends: R (>= 4.0.0)
Imports: SingleCellExperiment, SummarizedExperiment, S4Vectors, Rcpp
        (>= 1.0.7), cluster, utils, plot3D, ComplexHeatmap, circlize,
        grid, grDevices, graphics
Suggests: ggplot2, TENxPBMCData, dplyr, Rtsne, Seurat, AnnotationDbi,
        BiocGenerics, stringr, org.Hs.eg.db, knitr, rmarkdown, testthat
        (>= 3.0.0)
RoxygenNote: 7.1.2
LinkingTo: Rcpp
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/ASURAT
git_branch: RELEASE_3_22
git_last_commit: 9212396
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-30
NeedsCompilation: yes
Packaged: 2025-10-30 20:29:13 UTC; biocbuild
Author: Keita Iida [aut, cre] (ORCID: <https://orcid.org/0000-0002-1076-830X>),
  Johannes Nicolaus Wibisana [ctb]
Maintainer: Keita Iida <kiida@protein.osaka-u.ac.jp>
