# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Perform one-shot adjacent swapping for each element.
#'
#' @param listdata A list of vector and integer.
#'   
#' @return A List.
#' @export
#'
#' @examples
#' swap_pass(list(vec = c(1, 1, 0), cnt = 0))
#'
swap_pass <- function(listdata) {
    .Call('_ASURAT_swap_pass', PACKAGE = 'ASURAT', listdata)
}

#' Perform bubble sorting, counting the number of steps.
#'
#' @param listdata A list of vector and integer.
#'   For example, in R code, listdata = list(vec = c(1, 0, 1, ...), cnt = 0).
#'   The integer (cnt = 0) is the initial number of steps for bubble sorting.
#'
#' @return A List.
#' @export
#'
#' @examples
#' bubble_sort(list(vec = c(1, 1, 0), cnt = 0))
#'
bubble_sort <- function(listdata) {
    .Call('_ASURAT_bubble_sort', PACKAGE = 'ASURAT', listdata)
}

